@@ -5,7 +5,7 @@ use Data::Dumper ;
 use PBS::Output ;
 use PBS::Shell ;
 
-die "Old example kept for historical reasons only!\n This can be made to work again :)\n" ;
+die "Old example kept for historical reasons only!\n This can be made to work again :)\n" ;
 
 #-------------------------------------------------------------------------------
 sub BuildSequenceMiner
